-----------------------------------------------------------------------------------------------
-- 'MatchQ' v0.8a [13/02/2015]
-----------------------------------------------------------------------------------------------

 
require "Window"
require "GameLib"
require "MatchingGame"
require "GroupLib" 



-----------------------------------------------------------------------------------------------
-- MatchQ Module Definition
-----------------------------------------------------------------------------------------------
local MatchQ = {} 
local L = Apollo.GetPackage("Gemini:Locale-1.0").tPackage:GetLocale("MatchQ", true)
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------

local MQ_VER = "0.8a"


local tMQ = {	
            
      			
            alert = 	{
				       		[0] = L["MQ_ACTIVED"], 
					   		[1] = L["MQ_RELAUNCH"],
					   		[2] = L["MQ_DESERTER"], 
					   		[3] = L["MQ_GROUPNOTREADY"],
					   		[4] = L["MQ_DOSETMATCH"],
							[5] = L["MQ_MISSEDCALL"],

						
			     		},			 
    
		 
      		timer = 	{
	             			default = 1,   
	             			deserter = 0,
	              		},
         
			addon = 	{
	             			[1] = "MatchMaker",   
	             			[2] = "EasyMatchMaker",
	              		}


		   }       	

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function MatchQ:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
 
   
o.tMatchQ = {
			  enabled = true,
				leave = false,	
			}

			
    return o
end

function MatchQ:Init()
	Apollo.RegisterAddon(self, false)
end
 


-----------------------------------------------------------------------------------------------
-- OnLoad
-----------------------------------------------------------------------------------------------
function MatchQ:OnLoad()

	self.xmlDoc = XmlDoc.CreateFromFile("MatchQ.xml")
	
	Apollo.RegisterEventHandler("ChangeWorld", "OnChangeWorld", self) 
	Apollo.RegisterEventHandler("ToggleGroupFinder", "OnToggle", self)
	Apollo.RegisterEventHandler("MatchExited", "OnLeaveMatch", self)
	Apollo.RegisterEventHandler("MatchLeft", "OnLeaveMatch", self)	
	Apollo.RegisterEventHandler("MatchingGameReady", "OnMatchReady", self)
	Apollo.RegisterEventHandler("MatchingJoinQueue", "OnMatchJoin", self)
	Apollo.RegisterEventHandler("PVPMatchFinished", "OnMatchFinished", self)
	Apollo.RegisterEventHandler("MatchingCancelPendingGame", "OnMissedCall", self) 
			
	Apollo.RegisterSlashCommand("matchq", "OnSlash", self)	
	Apollo.RegisterSlashCommand("mq", "OnSlash", self)	
	
	Apollo.RegisterTimerHandler("Relaunch_Timer", "OnRelaunchTimer", self)
	Apollo.RegisterTimerHandler("Leave_Timer", "OnLeaveTimer", self)
	Apollo.RegisterTimerHandler("WaitLoad_Timer", "OnWaitLoadTimer", self)
    Apollo.RegisterTimerHandler("Panel_Hide_Timer", "OnPanelHideTimer", self)
			
	self.ComeFromMatch = false
	self.nIdAlert = 0 
end


-----------------------------------------------------------------------------------------------
-- OnToggle									
-----------------------------------------------------------------------------------------------
function MatchQ:OnToggle()
  
	if self.MatchQ_Btn == nil then 
						
		for _=1,#tMQ.addon do
			
			local aCheckAddon = Apollo.GetAddon(tMQ.addon[_])
			if aCheckAddon ~= nil then self.TargetAddon = aCheckAddon end	
		
		end
		
		self.MatchQ_Btn = Apollo.LoadForm(self.xmlDoc, "MatchQBtn", self.TargetAddon.wndMain, self)	
		self.MatchQ_Btn:SetTooltip("MatchQ v" .. MQ_VER)
		self.MatchQ_Btn:FindChild("MatchQTxt"):SetText(L["MQ_AUTOMQ"])
		self.MatchQ_Btn:FindChild("LeaveTxt"):SetText(L["MQ_LEAVE"])
		self.TargetAddon.wndQueueInfo:SetAnchorOffsets(35, -364, 299, -85)
	end
		
	self.MatchQ_Btn:SetCheck(self.tMatchQ.enabled)
end



-----------------------------------------------------------------------------------------------
-- OnMatchReady							
-----------------------------------------------------------------------------------------------
function MatchQ:OnMatchReady()

   if self.RememberMatchsSelected == nil then 

		self:OnToggle()
  		self.RememberMatchsSelected = self.TargetAddon.matchesQueued 
   else
   		self.RememberMatchsSelected = self.TargetAddon.matchesSelected
   end
	
   
   self.RememberModeSelected = self.TargetAddon.wndModeListToggle:GetText()

   self:MatchReadyAlert()

end



-----------------------------------------------------------------------------------------------
-- OnMatchJoin								
-----------------------------------------------------------------------------------------------
function MatchQ:OnMatchJoin()

	if self.TargetAddon == nil then return end

	self.RememberMatchsSelected = self.TargetAddon.matchesSelected
	self.RememberModeSelected = self.TargetAddon.wndModeListToggle:GetText()

	if not self.MatchQ_Btn:IsEnabled() or not self.tMatchQ.enabled then return end
	
	self:AlertStatus(self.nIdAlert)
	self.nIdAlert = 0
end



-----------------------------------------------------------------------------------------------
-- OnSlash
-----------------------------------------------------------------------------------------------
function MatchQ:OnSlash(strCommand)

	self.tMatchQ.enabled = not self.tMatchQ.enabled
	self:OnToggle()
end



----------------------------------------------------------------------------------------------
-- OnChangeWorld
-----------------------------------------------------------------------------------------------
function MatchQ:OnChangeWorld()

	self:OnToggle()

	if not self.MatchQ_Btn:IsEnabled() or not self.tMatchQ.enabled then return end
	if not self.ComeFromMatch then return end

	self.bLoaded = false
	self.ComeFromMatch = false
	
	Apollo.CreateTimer("WaitLoad_Timer", 2, true)
end



-----------------------------------------------------------------------------------------------
-- OnMatchFinish											 
-----------------------------------------------------------------------------------------------
function MatchQ:OnMatchFinished()

	if not self.tMatchQ.leave then return end

	Apollo.CreateTimer("Leave_Timer", 2, false)
end



-----------------------------------------------------------------------------------------------
-- OnMissedCall											 
-----------------------------------------------------------------------------------------------
function MatchQ:OnMissedCall()

	self:OnToggle()
	
    if not self.MatchQ_Btn:IsEnabled() or not self.tMatchQ.enabled then return end

    self.bLoaded = false
	self.bMissedCall = true
		
	Apollo.CreateTimer("WaitLoad_Timer", 4, false)
end



-----------------------------------------------------------------------------------------------
-- OnLeaveTimer												  
-----------------------------------------------------------------------------------------------
function MatchQ:OnLeaveTimer()
  	
	if MatchingGame.IsInMatchingGame() then
		MatchingGame.LeaveMatchingGame()
	end
end


-----------------------------------------------------------------------------------------------
-- OnLeaveMatch										
-----------------------------------------------------------------------------------------------
function MatchQ:OnLeaveMatch()
  
	self.ComeFromMatch = true
end



-----------------------------------------------------------------------------------------------
-- AsGroupRequirement			      	
-----------------------------------------------------------------------------------------------
function MatchQ:AsGroupRequirement()
  
    local bDoesGroupMeetRequirements = true

	for strMatchName, oMatch in pairs(self.TargetAddon.matchesSelected) do
			bDoesGroupMeetRequirements = bDoesGroupMeetRequirements and oMatch:DoesGroupMeetRequirements()
		end
		
	return bDoesGroupMeetRequirements		
end



-----------------------------------------------------------------------------------------------
-- OnWaitLoadTimer			
-----------------------------------------------------------------------------------------------
function MatchQ:OnWaitLoadTimer()

	if GameLib.IsCharacterLoaded() and not self.bLoaded and self.TargetAddon ~= nil then 

        self.uPlayer = GameLib.GetPlayerUnit() 				
        local nActualTimer = tMQ.timer.default 				
		local bIsDeserter = self:IsDeserter(self.uPlayer) 	
		local bInGroupLeader = self:IsInGroup(self.uPlayer) 
				
		if bInGroupLeader and MatchingGame.CanQueueAsGroup() and not self:AsGroupRequirement() then
			
			self:AlertStatus(3)
			return
		end
		
		
		if bIsDeserter then
			
			nActualTimer = tMQ.timer.deserter
			self:AlertStatus(2)
		end
		
				
        self.bLoaded = true 
		Apollo.CreateTimer("Relaunch_Timer", nActualTimer, false)
				
  	end	
end



-----------------------------------------------------------------------------------------------
-- AlertStatus						
-----------------------------------------------------------------------------------------------
function MatchQ:AlertStatus(id)
   
    if self.wndPanel ~= nil then self.wndPanel:Destroy() end


	self.tLines = { [1] = { sText = "\'MatchQ\'", sColor = "xkcdAzure", sFont = "CRB_Header20" },
	               	[2] = { sText = tMQ.alert[id] .. "..", sColor = "xkcdBeige", sFont = "CRB_Header14" },
				  }
	
	if id ~= 4 then self:GetMatchName() end 	
	
	self.wndPanel = Apollo.LoadForm(self.xmlDoc, "PanelWnd", nil, self)
	
	for idx = 1, #self.tLines do
		
		self.wndLine = Apollo.LoadForm(self.xmlDoc, "BaseLine", self.wndPanel:FindChild("LineWnd"), self)
		self.wndLine:SetFont(self.tLines[idx].sFont)
		self.wndLine:SetTextColor(self.tLines[idx].sColor)
		self.wndLine:SetText(self.tLines[idx].sText)
	end
	
	
	local nLineHeight = self.wndLine:GetHeight()
	local nHeight = (#self.tLines - 2) * nLineHeight
	local tSize = Apollo.GetDisplaySize()
	local nHw = tSize.nWidth 
	local nHh = tSize.nHeight
	local nHrw = self.wndPanel:GetWidth()
	local nHrh = nHeight + 80
	local nLeft = (nHw - nHrw) / 2
	local nTop = (nHh - nHrh) / 2 
	local nTop = nTop - 300

	self.wndPanel:FindChild("LineWnd"):ArrangeChildrenVert(0)
	self.wndPanel:SetAnchorOffsets(nLeft, nTop, nHrw + nLeft, nHrh + nTop)
	self.wndPanel:Show(true, false)
	
	Apollo.CreateTimer("Panel_Hide_Timer", 8, false)
end



-----------------------------------------------------------------------------------------------
-- OnPanelHideTimer							
-----------------------------------------------------------------------------------------------
function MatchQ:OnPanelHideTimer()
    
		if self.wndPanel then 
			self.wndPanel:Show(false, false)
			self.wndPanel:Destroy() 
		end
		
		self.tLines = nil
end 


-----------------------------------------------------------------------------------------------
-- GetMatchName						
-----------------------------------------------------------------------------------------------
function MatchQ:GetMatchName()
      
    table.insert(self.tLines, { sText = "\(".. self.RememberModeSelected .. "\)", sColor = "gray", sFont = "CRB_Header14" })
 	
	for _, uMatch in pairs(self.TargetAddon.matchesQueued) do    
	 
	   table.insert(self.tLines, { sText = _, sColor = "white", sFont = "CRB_Header14" })   
	end
end 


-----------------------------------------------------------------------------------------------
-- MatchReadyAlert						
-----------------------------------------------------------------------------------------------
function MatchQ:MatchReadyAlert()
    	
	Apollo.AlertAppWindow()

end 


-----------------------------------------------------------------------------------------------
-- IsDeserter					
-----------------------------------------------------------------------------------------------
function MatchQ:IsDeserter(unit)

	local uPlayer = unit    			
	local tBuff = uPlayer:GetBuffs() 	
	local tHarm = tBuff.arHarmful	 	

	for _=1, #tHarm do
      
		if tHarm[_].splEffect:GetId() == 45444 then 
				   
		    tMQ.timer.deserter = tHarm[_].fTimeRemaining    
		 
			return true
		end
	end

	return false
end


-----------------------------------------------------------------------------------------------
-- IsInGroup					
-----------------------------------------------------------------------------------------------
function MatchQ:IsInGroup(unit)

	local uPlayer = unit    						
	local sPlayerName = uPlayer:GetName()			
	local bInGroup = GroupLib.InGroup(sPlayerName)	
	local bGroupLeader = GroupLib.AmILeader()		
	
	if bInGroup	and bGroupLeader then return true end

	return false
end




-----------------------------------------------------------------------------------------------
-- OnRelaunchTimer  										
-----------------------------------------------------------------------------------------------
function MatchQ:OnRelaunchTimer()

   	Apollo.StopTimer("WaitLoad_Timer")
   	if not self.tMatchQ.enabled then return end

	local bInGroupLeader = self:IsInGroup(self.uPlayer)		
	local nIdAlert = 1 
	
	
    if self.RememberMatchsSelected == nil then 
		self:AlertStatus(4)
		return
    else
		if self.bMissedCall then 
			nIdAlert = 5 
			self.bMissedCall = false
		end 
		
		self.nIdAlert = nIdAlert 
		self.TargetAddon.matchesSelected = self.RememberMatchsSelected
   	end 

			
	if bInGroupLeader == false then			
		
		self.TargetAddon:OnJoin() 
	else
		
		self.TargetAddon:OnJoinAsGroup()
	end
	
	
end



---------------------------------------------------------------------------------------------------
-- OnCheckUncheck
---------------------------------------------------------------------------------------------------
function MatchQ:OnCheckUncheck( wndHandler, wndControl, eMouseButton )
    
	local cName = wndControl:GetName() 
	
	if cName == "MatchQBtn" then

		self.tMatchQ.enabled = wndControl:IsChecked()
		
		if not self.MatchQ_Btn:IsEnabled() or not self.tMatchQ.enabled then return end
		if MatchingGame.IsQueuedForMatching() or MatchingGame.IsQueuedAsGroup() then self:AlertStatus(0) end

	end
	
	
	if cName == "LeaveBtn" then

		self.tMatchQ.leave = wndControl:IsChecked()
	end
end



-----------------------------------------------------------------------------------------------
-- MatchQ Instance
-----------------------------------------------------------------------------------------------
local MatchQInst = MatchQ:new()
MatchQInst:Init()
